#ifndef _SERVICE_H_
#define _SERVICE_H_

/*
 * SysV service classes
 */

#include <qstring.h>
#include <qstringlist.h>
#include <qdict.h>
#include <qxml.h>

extern int srv_errno;

/*
 * system services 
 */
class Service {
public:
	Service(const QString& srvname);
	~Service();

	QString name;
	QString subject;
	QString description;
	QString progpath;
	QString type; /* oneshot, daemon or special */
	QString start_default_seq, kill_default_seq;
	QStringList progs; /* binaries */

	/* 
	 * running sequence [0..99]
	 * runseq[level][action]
	 * level: [0..6]
	 * action: 0: start, 1: kill
	 * if sequence = -1, means not run in that level.
	 */
	int default_seq[2];
	int runseq[7][2]; 
	/// int cur_runlevel;

	void print();
	int start(QString& output);
	int stop(QString& output);
	int restart(QString& output);
	int status(); /* 0: stopped, 1: running, -1: unknown */	
	int suspend();
	int resume();
	int set_progpath(const QString& ppath); /* only called by service XML parser */
	QString get_script(); /* load script contents */

	/* run level change functions */
	int set_mode_auto(int runlevel);
	int set_mode_manual(int runlevel);	
	int change_level_seq(int level, int new_seq, int action);
	int add_to_level(int level, int seq, int action);
	int remove_from_level(int level, int seq, int action);
 private:
	QString seq_to_str(int seq); /* convert sequence # to string */
};

/*
 * keep lists of services indexed by run level #
 * Note: it has not been used yet.
 */
class RunLevel {
public:
	int level;
	QPtrList<Service> startlist;
	QPtrList<Service> stoplist;
};

/*
 * System Service Manager
 * maintains a list of system services.
 */
class ServiceManager {
public:
	ServiceManager();
	~ServiceManager();
	void print();
	int get_cur_runlevel();
	Service* find_service(const QString& name);

	QPtrList<Service> services;
	int add_std_service(Service* s); /* ONLY called by XML parser */

private:
	QDict<Service> services_dict;
	QDict<Service> std_services;
	
	Service* load_service(const QString& filename, bool load_desc = false);
	int load_std_services();
	int print_std_services();
 	void load_level(int level);

	int current_runlevel();	
	int cur_runlevel;
};

#endif /* _SERVICE_H_ */
